function [ xx,yy,AA ] = DensityMap( x0,y,z,A,r,n )
% function [ xx,yy,AA ] = DensityMap( x,y,z,A,r,n )
% Density map of x-y-z space with densities A; r projizierend
%
% Funktion calculates the density of points with values A and coordinates
% x, y, z along the direction specified by r-vector. The resulting density
% map has a maximum of n points along the largest dimension.
%
% Input: ------------------------------------------------------------------
% x,y,z     coordinates, all the same size
% A         size(x), values on these points
% n         size 1, size(AA,longest dim.)
% Output: -----------------------------------------------------------------
% xx,yy     coordinates of the AA-map
% AA        map with density values

if nargin < 6
    n = 25;
elseif isempty(n)
    n = 25;
end

if nargin < 5
    r = [0 0 1];
elseif isempty(r)
    r = [0 0 1];
end

if nargin < 4
    A = 1;
elseif isempty(A)
    A = 1;
end

if isempty(z)
    z = zeros(size(x0));
end

x0 = x0(:);
y = y(:);
z = z(:);


%% rotation of x,y,z:
R = RMat3(r,-[0 0 1]); % rotation matrix
if vecnorm(R(1,1:2))>1e-12 % map of x(space) shall point along x(map)
    phi = atan2(R(1,2),R(1,1));
else % map of y(space) shall point along x(map)
    z = -z;
    phi = pi/2;
end
R = R*[cos(phi) -sin(phi) 0;sin(phi) cos(phi) 0;0 0 1];

x = x0*R(1,1)+y*R(2,1)+z*R(3,1);
y = x0*R(1,2)+y*R(2,2)+z*R(3,2);
clear x0

%% pixel width
ds = max([max(x)-min(x),max(y)-min(y)])/n;
x = x+ds*(rand(size(x))-0.5);
y = y+ds*(rand(size(y))-0.5);

%% index vector
xmin = min(x);
ymin = min(y);
n1 = ceil((max(x)-xmin)/ds)+2;
n2 = ceil((max(y)-ymin)/ds)+2;
x = (x-xmin)./ds;
y = (y-ymin)./ds;
z = floor(x)+floor(y)*n1+1;
x = x-floor(x);
y = y-floor(y);

%% density mapping; linear interpolation
AA = zeros(n1,n2);
if numel(A) == 1
    for k = 1:numel(z)
        j = z(k);
        AA(j) =      AA(j) +     (1-x(k))*(1-y(k))*A;
        AA(j+1) =    AA(j+1) +      x(k) *(1-y(k))*A;
        AA(j+n1) =   AA(j+n1) +  (1-x(k))*   y(k) *A;
        AA(j+n1+1) = AA(j+n1+1) +   x(k) *   y(k) *A;
    end
else
    for k = 1:numel(z)
        j = z(k);
        AA(j) =      AA(j) +     (1-x(k))*(1-y(k))*A(k);
        AA(j+1) =    AA(j+1) +      x(k) *(1-y(k))*A(k);
        AA(j+n1) =   AA(j+n1) +  (1-x(k))*   y(k) *A(k);
        AA(j+n1+1) = AA(j+n1+1) +   x(k) *   y(k) *A(k);
    end
end

%% x -> -yy; y -> xx
xx =   (ds*(0:n1-1)'*ones(1,n2)+xmin);
yy =   (ds*ones(n1,1)*(0:n2-1) +ymin);
AA = smooth(AA,1.2);

end

function map = smooth(map,s)
    daa = ceil(4*s);
    dxi = abs(ones(2*daa+1,1).*(-daa:daa));
    dyi = abs((-daa:daa)'.*ones(1,2*daa+1));
    Kernel = sqrt(exp(-(dxi.^2+dyi.^2)/s.^2));
    Kernel = Kernel/sum(Kernel(:));
    map = filter2(Kernel,map);
end

